@@ -188,8 +188,9 @@ vector<DNSResourceRecord*> DNSPacket::getAPRecords()
       ++i)
     {
       if(i->d_place!=DNSResourceRecord::ADDITIONAL && 
-         (i->qtype.getCode()==15 || 
-          i->qtype.getCode()==2 )) // CNAME or MX or NS
+         (i->qtype.getCode()==QType::MX ||
+          i->qtype.getCode()==QType::NS ||
+          i->qtype.getCode()==QType::SRV))
         {
           arrs.push_back(&*i);
         }
@@ -226,6 +227,17 @@ bool DNSPacket::couldBeCached()
   return d_ednsping.empty() && !d_wantsnsid && qclass==QClass::IN;
 }
 
+unsigned int DNSPacket::getMinTTL()
+{
+  unsigned int minttl = UINT_MAX;
+  BOOST_FOREACH(DNSResourceRecord rr, d_rrs) {
+  if (rr.ttl < minttl)
+      minttl = rr.ttl;
+  }
+
+  return minttl;
+}
+
 /** Must be called before attempting to access getData(). This function stuffs all resource
  *  records found in rrs into the data buffer. It also frees resource records queued for us.
  */
@@ -533,6 +545,11 @@ void DNSPacket::setRemote(const ComboAddress *s)
   d_remote=*s;
 }
 
+bool DNSPacket::hasEDNSSubnet()
+{
+  return d_haveednssubnet;
+}
+
 Netmask DNSPacket::getRealRemote() const
 {
   if(d_haveednssubnet)
