@@ -17,9 +17,9 @@
 #include <boost/multi_index/key_extractors.hpp>
 #include <cctype>
 
-using namespace boost;
+#include "namespaces.hh"
 using namespace ::boost::multi_index;
-using namespace std;
+#include "namespaces.hh"
 StatBag S;
 
 int main(int argc, char** argv)
@@ -29,7 +29,7 @@ try
 
   /*
   IPEndpoint remote(argc > 2 ? argv[2] : "127.0.0.1", 
-		    argc > 3 ? atoi(argv[3]) : 5300);
+        	    argc > 3 ? atoi(argv[3]) : 5300);
 
   */
 
@@ -47,28 +47,28 @@ try
     
     while(pr.getUDPPacket()) {
       try {
-	MOADNSParser mdp((const char*)pr.d_payload, pr.d_len);
-	if(mdp.d_qtype < 256)
-	  counts[mdp.d_qtype]++;
+        MOADNSParser mdp((const char*)pr.d_payload, pr.d_len);
+        if(mdp.d_qtype < 256)
+          counts[mdp.d_qtype]++;
 
-	for(int i=0; i < mdp.d_qname.length(); ++i)
-	  if(!isalnum(mdp.d_qname[i]) && mdp.d_qname[i]!='.' && mdp.d_qname[i]!='-' && mdp.d_qname[i]!='_') {
-	    //	  cout<<mdp.d_qname<<"|"<<mdp.d_qtype<<"|"<<mdp.d_qclass<<"\n";
-	    // sock.sendTo(string(pr.d_payload, pr.d_payload + pr.d_len), remote);
-	    break;
-	  }
-	if(mdp.d_qtype > 256 || mdp.d_qclass!=1 ) {
-	  //	sock.sendTo(string(pr.d_payload, pr.d_payload + pr.d_len), remote);
-	  
-	}
-	for(MOADNSParser::answers_t::const_iterator i=mdp.d_answers.begin(); i!=mdp.d_answers.end(); ++i) {          
-	  
-	}
-	
+        for(int i=0; i < mdp.d_qname.length(); ++i)
+          if(!isalnum(mdp.d_qname[i]) && mdp.d_qname[i]!='.' && mdp.d_qname[i]!='-' && mdp.d_qname[i]!='_') {
+            //	  cout<<mdp.d_qname<<"|"<<mdp.d_qtype<<"|"<<mdp.d_qclass<<"\n";
+            // sock.sendTo(string(pr.d_payload, pr.d_payload + pr.d_len), remote);
+            break;
+          }
+        if(mdp.d_qtype > 256 || mdp.d_qclass!=1 ) {
+          //	sock.sendTo(string(pr.d_payload, pr.d_payload + pr.d_len), remote);
+          
+        }
+        for(MOADNSParser::answers_t::const_iterator i=mdp.d_answers.begin(); i!=mdp.d_answers.end(); ++i) {          
+          
+        }
+        
       }
       catch(MOADNSException &e) {
-	cout<<"Error from remote "<<U32ToIP(ntohl(*((uint32_t*)&pr.d_ip->ip_src)))<<": "<<e.what()<<"\n";
-	//	sock.sendTo(string(pr.d_payload, pr.d_payload + pr.d_len), remote);
+        cout<<"Error from remote "<<U32ToIP(ntohl(*((uint32_t*)&pr.d_ip->ip_src)))<<": "<<e.what()<<"\n";
+        //	sock.sendTo(string(pr.d_payload, pr.d_payload + pr.d_len), remote);
       }
     }
   }
