@@ -41,7 +41,7 @@ public:
     
   void lookup(const QType &type, const string &qdomain, DNSPacket *p, int zoneId)
   {
-    if((type.getCode()!=QType::ANY && type.getCode()!=QType::A) || !pdns_iequals(qdomain, d_ourname))  // we only know about random.powerdns.com A
+    if((type.getCode()!=QType::ANY && type.getCode()!=QType::A) || !pdns_iequals(qdomain, d_ourname))  // we only know about random.example.com A by default
       d_answer="";                                                  // no answer
     else {
       ostringstream os;
