@@ -5,36 +5,14 @@
 #include "dnsrecords.hh"
 #include "arguments.hh"
 #include "syncres.hh"
+#include "recursor_cache.hh"
+#include "cachecleaner.hh"
 
-using namespace std;
-using namespace boost;
+#include "namespaces.hh"
+#include "namespaces.hh"
 
 #include "config.h"
 
-#ifdef GCC_SKIP_LOCKING
-#include <bits/atomicity.h>
-// This code is ugly but does speedup the recursor tremendously on multi-processor systems, and even has a large effect (20, 30%) on uniprocessor 
-namespace __gnu_cxx
-{
-  _Atomic_word
-  __attribute__ ((__unused__))
-  __exchange_and_add(volatile _Atomic_word* __mem, int __val)
-  {
-    register _Atomic_word __result=*__mem;
-    *__mem+=__val;
-    return __result;
-  }
-
-  void
-  __attribute__ ((__unused__))
-  __atomic_add(volatile _Atomic_word* __mem, int __val)
-  {
-    *__mem+=__val;
-  }
-}
-#endif
-
-
 DNSResourceRecord String2DNSRR(const string& qname, const QType& qt, const string& serial, uint32_t ttd)
 {
   DNSResourceRecord rr;
@@ -49,6 +27,7 @@ DNSResourceRecord String2DNSRR(const string& qname, const QType& qt, const strin
   }
   else if(rr.qtype.getCode()==QType::AAAA && serial.size()==16) {
     ComboAddress tmp;
+    memset(&tmp, 0, sizeof(tmp));
     tmp.sin4.sin_family=AF_INET6;
     memcpy(tmp.sin6.sin6_addr.s6_addr, serial.c_str(), 16);
     rr.content=tmp.toString();
@@ -59,12 +38,12 @@ DNSResourceRecord String2DNSRR(const string& qname, const QType& qt, const strin
 
     while((labellen=serial.at(frompos++))) {
       if((labellen & 0xc0) == 0xc0) {
-	string encoded=simpleCompress(qname);
-	uint16_t offset=256*(labellen & ~0xc0) + (unsigned int)serial.at(frompos++) - sizeof(dnsheader)-5;
+        string encoded=simpleCompress(qname);
+        uint16_t offset=256*(labellen & ~0xc0) + (unsigned int)serial.at(frompos++) - sizeof(dnsheader)-5;
 
-	simpleExpandTo(encoded, offset, rr.content);
-	//	cerr<<"Oops, fallback, content so far: '"<<rr.content<<"', offset: "<<offset<<", '"<<qname<<"', "<<qt.getName()<<"\n";
-	break;
+        simpleExpandTo(encoded, offset, rr.content);
+        //	cerr<<"Oops, fallback, content so far: '"<<rr.content<<"', offset: "<<offset<<", '"<<qname<<"', "<<qt.getName()<<"\n";
+        break;
       }
       rr.content.append(serial.c_str()+frompos, labellen);
       frompos+=labellen;
@@ -82,6 +61,7 @@ DNSResourceRecord String2DNSRR(const string& qname, const QType& qt, const strin
   return rr;
 }
 
+// returns the RDATA for rr - might be compressed!
 string DNSRR2String(const DNSResourceRecord& rr)
 {
   uint16_t type=rr.qtype.getCode();
@@ -116,6 +96,7 @@ unsigned int MemRecursorCache::bytes()
   unsigned int ret=0;
 
   for(cache_t::const_iterator i=d_cache.begin(); i!=d_cache.end(); ++i) {
+    ret+=sizeof(struct CacheEntry);
     ret+=(unsigned int)i->d_qname.length();
     for(vector<StoredRecord>::const_iterator j=i->d_records.begin(); j!= i->d_records.end(); ++j)
       ret+=j->size();
@@ -123,83 +104,20 @@ unsigned int MemRecursorCache::bytes()
   return ret;
 }
 
-int MemRecursorCache::getDirect(time_t now, const char* qname, const QType& qt, uint32_t ttd[10], char* data[10], uint16_t len[10])
-{
-  if(!d_cachecachevalid || Utility::strcasecmp(d_cachedqname.c_str(), qname)) {
-//    cerr<<"had cache cache miss for '"<<qname<<"'"<<endl;
-    d_cachedqname=qname;
-    d_cachecache=d_cache.equal_range(tie(qname));
-    d_cachecachevalid=true;
-  }
-  else
-    ;
-  //    cerr<<"had cache cache hit!"<<endl;
-
-  if(d_cachecache.first == d_cachecache.second) {
-    g_stats.noShuntNoMatch++;
-    return false;
-  }
-
-  pair<cache_t::iterator, cache_t::iterator> range = d_cachecache;
-  
-  unsigned int n=0;
-  for(;range.first != range.second; ++range.first) {
-    if(range.first->d_qtype == QType::CNAME) { // if we see a cname, we need the whole shebang (for now)
-      g_stats.noShuntCNAME++;
-      return false;
-    }
-    if(range.first->d_qtype != qt.getCode())
-      continue;
-    if(range.first->getTTD() < (unsigned int) now) {
-      g_stats.noShuntExpired++;
-      return false;
-    }
-    
-    if(range.first->d_records.empty() || range.first->d_records.size() > 9 ) {
-      g_stats.noShuntSize++;
-      return false;
-    }
-    
-    size_t limit=range.first->d_records.size();
-    n=0;
-    for(; n < limit; ++n) {
-      data[n]=(char*)range.first->d_records[n].d_string.c_str();
-      len[n]=range.first->d_records[n].d_string.length();
-      ttd[n]=range.first->d_records[n].d_ttd;
-    }
-    if(n<10) {
-      data[n]=0;
-      typedef cache_t::nth_index<1>::type sequence_t;
-      sequence_t& sidx=d_cache.get<1>();
-      sequence_t::iterator si=d_cache.project<1>(range.first);
-      sidx.relocate(sidx.end(), si); // move it in the LRU list 
-      // can't yet return, need to figure out if there isn't a CNAME that messes things up
-    }
-    else
-      return false;
-  }
-  if(!n)
-    g_stats.noShuntNoMatch++;
-  return n;
-
-}
-
 int MemRecursorCache::get(time_t now, const string &qname, const QType& qt, set<DNSResourceRecord>* res)
 {
   unsigned int ttd=0;
-
   //  cerr<<"looking up "<< qname+"|"+qt.getName()<<"\n";
 
-  if(!d_cachecachevalid || Utility::strcasecmp(d_cachedqname.c_str(), qname.c_str())) {
+  if(!d_cachecachevalid || !pdns_iequals(d_cachedqname, qname)) {
     //    cerr<<"had cache cache miss"<<endl;
     d_cachedqname=qname;
     d_cachecache=d_cache.equal_range(tie(qname));
     d_cachecachevalid=true;
   }
   else
+    //    cerr<<"had cache cache hit!"<<endl;
     ;
-  //    cerr<<"had cache cache hit!"<<endl;
-
 
   if(res)
     res->clear();
@@ -207,29 +125,25 @@ int MemRecursorCache::get(time_t now, const string &qname, const QType& qt, set<
   if(d_cachecache.first!=d_cachecache.second) { 
     for(cache_t::const_iterator i=d_cachecache.first; i != d_cachecache.second; ++i) 
       if(i->d_qtype == qt.getCode() || qt.getCode()==QType::ANY || 
-	 (qt.getCode()==QType::ADDR && (i->d_qtype == QType::A || i->d_qtype == QType::AAAA) )
-	 ) {
-	typedef cache_t::nth_index<1>::type sequence_t;
-	sequence_t& sidx=d_cache.get<1>();
-	sequence_t::iterator si=d_cache.project<1>(i);
-	
-	for(vector<StoredRecord>::const_iterator k=i->d_records.begin(); k != i->d_records.end(); ++k) {
-	  if(k->d_ttd < 1000000000 || k->d_ttd > (uint32_t) now) {
-	    ttd=k->d_ttd;
-	    if(res) {
-	      DNSResourceRecord rr=String2DNSRR(qname, QType(i->d_qtype),  k->d_string, ttd); 
-	      res->insert(rr);
-	    }
-	  }
-	}
-	if(res) {
-	  if(res->empty())
-	    sidx.relocate(sidx.begin(), si); 
-	  else
-	    sidx.relocate(sidx.end(), si); 
-	}
-	if(qt.getCode()!=QType::ANY && qt.getCode()!=QType::ADDR) // normally if we have a hit, we are done
-	  break;
+         (qt.getCode()==QType::ADDR && (i->d_qtype == QType::A || i->d_qtype == QType::AAAA) )
+         ) {     
+        for(vector<StoredRecord>::const_iterator k=i->d_records.begin(); k != i->d_records.end(); ++k) {
+          if(k->d_ttd < 1000000000 || k->d_ttd > (uint32_t) now) {  // FIXME what does the 100000000 number mean?
+            ttd=k->d_ttd;
+            if(res) {
+              DNSResourceRecord rr=String2DNSRR(qname, QType(i->d_qtype),  k->d_string, ttd); 
+              res->insert(rr);
+            }
+          }
+        }
+        if(res) {
+          if(res->empty())
+            moveCacheItemToFront(d_cache, i);
+          else
+            moveCacheItemToBack(d_cache, i);
+        }
+        if(qt.getCode()!=QType::ANY && qt.getCode()!=QType::ADDR) // normally if we have a hit, we are done
+          break;
       }
 
     //    cerr<<"time left : "<<ttd - now<<", "<< (res ? res->size() : 0) <<"\n";
@@ -237,32 +151,34 @@ int MemRecursorCache::get(time_t now, const string &qname, const QType& qt, set<
   }
   return -1;
 }
+
+
  
 bool MemRecursorCache::attemptToRefreshNSTTL(const QType& qt, const set<DNSResourceRecord>& content, const CacheEntry& stored)
 {
   if(!stored.d_auth) {
-//    cerr<<"feel free to scribble non-auth data!"<<endl;
+    //~ cerr<<"feel free to scribble non-auth data!"<<endl;
     return false;
   }
 
   if(qt.getCode()!=QType::NS) {
-  //  cerr<<"Not NS record"<<endl;
+    //~ cerr<<"Not NS record"<<endl;
     return false;
   }
   if(content.size()!=stored.d_records.size()) {
-  //  cerr<<"Not equal number of records"<<endl;
+    //~ cerr<<"Not equal number of records"<<endl;
     return false;
   }
   if(stored.d_records.empty())
     return false;
 
   if(stored.d_records.begin()->d_ttd > content.begin()->ttl) {
-    // cerr<<"attempt to LOWER TTL - fine by us"<<endl;
+    //~ cerr<<"attempt to LOWER TTL - fine by us"<<endl;
     return false;
   }
 
 
- // cerr<<"Returning true - update attempt!\n";
+//  cerr<<"Returning true - update attempt!\n";
   return true;
 }
 
@@ -275,42 +191,50 @@ void MemRecursorCache::replace(time_t now, const string &qname, const QType& qt,
   tuple<string, uint16_t> key=make_tuple(qname, qt.getCode());
   cache_t::iterator stored=d_cache.find(key);
 
-  //  cerr<<"storing "<< qname+"|"+qt.getName()<<" -> '"<<content.begin()->content<<"'\n";
-
   bool isNew=false;
   if(stored == d_cache.end()) {
     stored=d_cache.insert(CacheEntry(key,vector<StoredRecord>(), auth)).first;
     isNew=true;
   }
-  
   pair<vector<StoredRecord>::iterator, vector<StoredRecord>::iterator> range;
 
   StoredRecord dr;
   CacheEntry ce=*stored;
 
-  if(qt.getCode()==QType::SOA || qt.getCode()==QType::CNAME)  // you can only have one (1) each of these
-    ce.d_records.clear();
+  //~ cerr<<"asked to store "<< qname+"|"+qt.getName()<<" -> '"<<content.begin()->content<<"', isnew="<<isNew<<", auth="<<auth<<", ce.auth="<<ce.d_auth<<"\n";
 
-  if(auth && !attemptToRefreshNSTTL(qt, content, ce) ) {
-    ce.d_records.clear(); // clear non-auth data
-    ce.d_auth = true;
-    isNew=true;           // data should be sorted again
+  if(qt.getCode()==QType::SOA || qt.getCode()==QType::CNAME)  { // you can only have one (1) each of these
+    //    cerr<<"\tCleaning out existing store because of SOA and CNAME\n";
+    ce.d_records.clear();
   }
 
   if(!auth && ce.d_auth) {  // unauth data came in, we have some auth data, but is it fresh?
     vector<StoredRecord>::iterator j;
     for(j = ce.d_records.begin() ; j != ce.d_records.end(); ++j) 
       if((time_t)j->d_ttd > now) 
-	break;
+        break;
     if(j != ce.d_records.end()) { // we still have valid data, ignore unauth data
+      //      cerr<<"\tStill hold valid auth data, and the new data is unauth, return\n";
       return;
     }
     else {
       ce.d_auth = false;  // new data won't be auth
     }
   }
+  
+  // make sure that we CAN refresh the root
+  if(auth && ((qname.length()==1 && qname[0]=='.') || !attemptToRefreshNSTTL(qt, content, ce) ) ) {
+    // cerr<<"\tGot auth data, and it was not refresh attempt of an NS record, nuking storage"<<endl;
+    ce.d_records.clear(); // clear non-auth data
+    ce.d_auth = true;
+    isNew=true;           // data should be sorted again
+  }
+  else
+    ; //cerr<<"\tNot nuking"<<endl;
 
+  // cerr<<"\tHave "<<content.size()<<" records to store\n";
   for(set<DNSResourceRecord>::const_iterator i=content.begin(); i != content.end(); ++i) {
+    // cerr<<"To store: "<<i->content<<endl;
     dr.d_ttd=i->ttl;
     dr.d_string=DNSRR2String(*i);
     
@@ -318,24 +242,34 @@ void MemRecursorCache::replace(time_t now, const string &qname, const QType& qt,
       ce.d_records.push_back(dr);
     else {
       range=equal_range(ce.d_records.begin(), ce.d_records.end(), dr);
-      
+
       if(range.first != range.second) {
-	for(vector<StoredRecord>::iterator j=range.first ; j!=range.second; ++j) {
-	  /* see http://mailman.powerdns.com/pipermail/pdns-users/2006-May/003413.html */
-	  if(j->d_ttd > (unsigned int) now && i->ttl > j->d_ttd && qt.getCode()==QType::NS && auth) // don't allow auth servers to *raise* TTL of an NS record
-	    continue;
-	  if(i->ttl > j->d_ttd || (auth && d_followRFC2181) ) // authoritative packets can override the TTL to be lower
-	    j->d_ttd=i->ttl;
-	}
+       // cerr<<"\t\tMay need to modify TTL of stored record\n";
+        for(vector<StoredRecord>::iterator j=range.first ; j!=range.second; ++j) {
+          /* see http://mailman.powerdns.com/pipermail/pdns-users/2006-May/003413.html */
+          if(j->d_ttd > (unsigned int) now && i->ttl > j->d_ttd && qt.getCode()==QType::NS && auth) { // don't allow auth servers to *raise* TTL of an NS record
+            //~ cerr<<"\t\tNot doing so, trying to raise TTL NS\n";
+            continue;
+          }
+          if(i->ttl > j->d_ttd || (auth && d_followRFC2181) ) { // authoritative packets can override the TTL to be lower
+            //~ cerr<<"\t\tUpdating the ttl, diff="<<j->d_ttd - i->ttl<<endl;;
+            j->d_ttd=i->ttl;
+          }
+          else {
+            //~ cerr<<"\t\tNOT updating the ttl, old= " <<j->d_ttd - now <<", new: "<<i->ttl - now <<endl;
+          }
+        }
       }
       else {
-	ce.d_records.push_back(dr);
-	sort(ce.d_records.begin(), ce.d_records.end());
+        //~ cerr<<"\t\tThere was no exact copy of this record, so adding & sorting\n";
+        ce.d_records.push_back(dr);
+        sort(ce.d_records.begin(), ce.d_records.end());
       }
     }
   }
 
   if(isNew) {
+    //    cerr<<"\tSorting (because of isNew)\n";
     sort(ce.d_records.begin(), ce.d_records.end());
   }
   
@@ -390,91 +324,39 @@ bool MemRecursorCache::doAgeCache(time_t now, const string& name, uint16_t qtype
 }
 
 
-void MemRecursorCache::doDumpAndClose(int fd)
+uint64_t MemRecursorCache::doDump(int fd)
 {
-  FILE* fp=fdopen(fd, "w");
-  if(!fp) {
-    close(fd);
-    return;
+  FILE* fp=fdopen(dup(fd), "w");
+  if(!fp) { // dup probably failed
+    return 0;
   }
-
+  fprintf(fp, "; main record cache dump from thread follows\n;\n");
   typedef cache_t::nth_index<1>::type sequence_t;
   sequence_t& sidx=d_cache.get<1>();
 
+  uint64_t count=0;
   time_t now=time(0);
   for(sequence_t::const_iterator i=sidx.begin(); i != sidx.end(); ++i) {
     for(vector<StoredRecord>::const_iterator j=i->d_records.begin(); j != i->d_records.end(); ++j) {
+      count++;
       try {
-	DNSResourceRecord rr=String2DNSRR(i->d_qname, QType(i->d_qtype), j->d_string, j->d_ttd - now);
-	fprintf(fp, "%s %d IN %s %s\n", rr.qname.c_str(), rr.ttl, rr.qtype.getName().c_str(), rr.content.c_str());
+        DNSResourceRecord rr=String2DNSRR(i->d_qname, QType(i->d_qtype), j->d_string, j->d_ttd - now);
+        fprintf(fp, "%s %d IN %s %s\n", rr.qname.c_str(), rr.ttl, rr.qtype.getName().c_str(), rr.content.c_str());
       }
       catch(...) {
-	fprintf(fp, "; error printing '%s'\n", i->d_qname.c_str());
+        fprintf(fp, "; error printing '%s'\n", i->d_qname.c_str());
       }
     }
   }
   fclose(fp);
-}
-
-void MemRecursorCache::doSlash(int perc)
-{
-  doPrune();
+  return count;
 }
 
 void MemRecursorCache::doPrune(void)
 {
-  uint32_t now=(uint32_t)time(0);
   d_cachecachevalid=false;
 
-  unsigned int maxCached=::arg().asNum("max-cache-entries");
-  unsigned int toTrim=0;
-  
-  unsigned int cacheSize=d_cache.size();
-
-  if(maxCached && cacheSize > maxCached) {
-    toTrim = cacheSize - maxCached;
-  }
-
-  //  cout<<"Need to trim "<<toTrim<<" from cache to meet target!\n";
-
-  typedef cache_t::nth_index<1>::type sequence_t;
-  sequence_t& sidx=d_cache.get<1>();
-
-  unsigned int tried=0, lookAt, erased=0;
-
-  // two modes - if toTrim is 0, just look through 10000 records and nuke everything that is expired
-  // otherwise, scan first 5*toTrim records, and stop once we've nuked enough
-  if(toTrim)
-    lookAt=5*toTrim;
-  else
-    lookAt=cacheSize/10;
-
-
-  sequence_t::iterator iter=sidx.begin(), eiter;
-  for(; iter != sidx.end() && tried < lookAt ; ++tried) {
-    unsigned int ttd=iter->getTTD();
-    if(ttd < now) { 
-      sidx.erase(iter++);
-      erased++;
-    }
-    else
-      ++iter;
-
-    if(toTrim && erased > toTrim)
-      break;
-  }
-
-  //  cout<<"erased "<<erased<<" records based on ttd\n";
-  
-  if(erased >= toTrim)
-    return;
-
-  //  if(toTrim)
-  //    cout<<"Still have "<<toTrim - erased<<" entries left to erase to meet target\n";
-
-
-  eiter=iter=sidx.begin();
-  std::advance(eiter, toTrim);
-  sidx.erase(iter, eiter);
+  unsigned int maxCached=::arg().asNum("max-cache-entries") / g_numThreads;
+  pruneCollection(d_cache, maxCached);
 }
 
