@@ -1,6 +1,6 @@
 /*
     PowerDNS Versatile Database Driven Nameserver
-    Copyright (C) 2002-2007  PowerDNS.COM BV
+    Copyright (C) 2002-2010  PowerDNS.COM BV
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -90,14 +90,14 @@ void *WebServer::serveConnection(void *p)
     map<string,string>varmap;
 
     for(vector<string>::const_iterator i=variables.begin();
-	i!=variables.end();++i) {
+        i!=variables.end();++i) {
 
       parts.clear();
       stringtok(parts,*i,"=");
       if(parts.size()>1)
-	varmap[parts[0]]=parts[1];
+        varmap[parts[0]]=parts[1];
       else
-	varmap[parts[0]]="";
+        varmap[parts[0]]="";
 
     }
 
@@ -110,16 +110,16 @@ void *WebServer::serveConnection(void *p)
 
       //      L<<Logger::Error<<"got line: '"<<line<<"'"<<endl;
       if(!toLower(line).find("authorization: basic ")) {
-	string cookie=line.substr(21);
-	string plain;
+        string cookie=line.substr(21);
+        string plain;
 
-	B64Decode(cookie,plain);
-	vector<string>cparts;
-	stringtok(cparts,plain,":");
-	//	L<<Logger::Error<<"Entered password: '"<<cparts[1].c_str()<<"', should be '"<<d_password.c_str()<<"'"<<endl;
-	if(cparts.size()==2 && !strcmp(cparts[1].c_str(),d_password.c_str())) { // this gets rid of terminating zeros
-	  authOK=1;
-	}
+        B64Decode(cookie,plain);
+        vector<string>cparts;
+        stringtok(cparts,plain,":");
+        //	L<<Logger::Error<<"Entered password: '"<<cparts[1].c_str()<<"', should be '"<<d_password.c_str()<<"'"<<endl;
+        if(cparts.size()==2 && !strcmp(cparts[1].c_str(),d_password.c_str())) { // this gets rid of terminating zeros
+          authOK=1;
+        }
       }
     }while(!line.empty());
 
@@ -137,14 +137,14 @@ void *WebServer::serveConnection(void *p)
     }
 
     HandlerFunction *fptr;
-    if((fptr=d_functions[baseUrl])) {
+    if(d_functions.count(baseUrl) && (fptr=d_functions[baseUrl])) {
       bool custom=false;
       string ret=(*fptr)(varmap, d_that, &custom);
 
       if(!custom) {
-	client->putLine("HTTP/1.1 200 OK\n");
-	client->putLine("Connection: close\n");
-	client->putLine("Content-type: text/html\n\n");
+        client->putLine("HTTP/1.1 200 OK\n");
+        client->putLine("Connection: close\n");
+        client->putLine("Content-type: text/html\n\n");
       }
       client->putLine(ret);
     }
@@ -190,19 +190,26 @@ WebServer::WebServer(const string &listenaddress, int port, const string &passwo
   d_listenaddress=listenaddress;
   d_port=port;
   d_password=password;
+  d_server = 0; // on exception, this class becomes a NOOP later on
+  try {
+    d_server = new Server(d_port, d_listenaddress);
+  }
+  catch(SessionException &e) {
+    L<<Logger::Error<<"Fatal error in webserver: "<<e.reason<<endl;
+  }
 }
 
 void WebServer::go()
 {
+  if(!d_server)
+    return;
   try {
-    Server *s=new Server(d_port, d_listenaddress);
-    
     Session *client;
     pthread_t tid;
     
     L<<Logger::Error<<"Launched webserver on "<<d_listenaddress<<":"<<d_port<<endl;
 
-    while((client=s->accept())) {
+    while((client=d_server->accept())) {
       pthread_create(&tid, 0 , &serveConnection, (void *)client);
     }
   }
